@@ -13,6 +13,7 @@
 #  limitations under the License.
 
 import warnings
+from typing import Optional, Tuple, Union
 
 import torch
 import torch.nn as nn
@@ -22,7 +23,7 @@ import poptorch
 from optimum.utils import logging
 from transformers import T5ForConditionalGeneration
 from transformers.modeling_outputs import BaseModelOutput, Seq2SeqLMOutput
-from transformers.models.t5.modeling_t5 import __HEAD_MASK_WARNING_MSG, T5LayerNorm, T5Stack
+from transformers.models.t5.modeling_t5 import __HEAD_MASK_WARNING_MSG, T5Block, T5Stack
 
 from ...generation_utils import IPUGenerationMixin
 from ...modeling_utils import (
@@ -39,6 +40,112 @@ from ...modeling_utils import (
 logger = logging.get_logger(__name__)
 
 
+class CustomT5Block(T5Block):
+    def forward(
+        self,
+        hidden_states,
+        attention_mask=None,
+        position_bias=None,
+        encoder_hidden_states=None,
+        encoder_attention_mask=None,
+        encoder_decoder_position_bias=None,
+        layer_head_mask=None,
+        cross_attn_layer_head_mask=None,
+        past_key_value=None,
+        use_cache=False,
+        output_attentions=False,
+        return_dict=True,
+    ):
+
+        if past_key_value is not None:
+            if not self.is_decoder:
+                logger.warning("`past_key_values` is passed to the encoder. Please make sure this is intended.")
+            expected_num_past_key_values = 2 if encoder_hidden_states is None else 4
+
+            if len(past_key_value) != expected_num_past_key_values:
+                raise ValueError(
+                    f"There should be {expected_num_past_key_values} past states. "
+                    f"{'2 (past / key) for cross attention. ' if expected_num_past_key_values == 4 else ''}"
+                    f"Got {len(past_key_value)} past key / value states"
+                )
+
+            self_attn_past_key_value = past_key_value[:2]
+            cross_attn_past_key_value = past_key_value[2:]
+        else:
+            self_attn_past_key_value, cross_attn_past_key_value = None, None
+
+        self_attention_outputs = self.layer[0](
+            hidden_states,
+            attention_mask=attention_mask,
+            position_bias=position_bias,
+            layer_head_mask=layer_head_mask,
+            past_key_value=self_attn_past_key_value,
+            use_cache=use_cache,
+            output_attentions=output_attentions,
+        )
+        hidden_states, present_key_value_state = self_attention_outputs[:2]
+        attention_outputs = self_attention_outputs[2:]  # Keep self-attention outputs and relative position weights
+
+        # clamp inf values to enable fp16 training
+        # Custom: Remove check for inf
+        if hidden_states.dtype == torch.float16:
+            clamp_value = torch.finfo(hidden_states.dtype).max - 1000
+            hidden_states = torch.clamp(hidden_states, min=-clamp_value, max=clamp_value)
+
+        do_cross_attention = self.is_decoder and encoder_hidden_states is not None
+        if do_cross_attention:
+            # the actual query length is unknown for cross attention
+            # if using past key value states. Need to inject it here
+            if present_key_value_state is not None:
+                query_length = present_key_value_state[0].shape[2]
+            else:
+                query_length = None
+
+            cross_attention_outputs = self.layer[1](
+                hidden_states,
+                key_value_states=encoder_hidden_states,
+                attention_mask=encoder_attention_mask,
+                position_bias=encoder_decoder_position_bias,
+                layer_head_mask=cross_attn_layer_head_mask,
+                past_key_value=cross_attn_past_key_value,
+                query_length=query_length,
+                use_cache=use_cache,
+                output_attentions=output_attentions,
+            )
+            hidden_states = cross_attention_outputs[0]
+
+            # clamp inf values to enable fp16 training
+            # Custom: Remove check for inf
+            if hidden_states.dtype == torch.float16:
+                clamp_value = torch.finfo(hidden_states.dtype).max - 1000
+                hidden_states = torch.clamp(hidden_states, min=-clamp_value, max=clamp_value)
+
+            # Combine self attn and cross attn key value states
+            if present_key_value_state is not None:
+                present_key_value_state = present_key_value_state + cross_attention_outputs[1]
+
+            # Keep cross-attention outputs and relative position weights
+            attention_outputs = attention_outputs + cross_attention_outputs[2:]
+
+        # Apply Feed Forward layer
+        hidden_states = self.layer[-1](hidden_states)
+
+        # clamp inf values to enable fp16 training
+        # Custom: Remove check for inf
+        if hidden_states.dtype == torch.float16:
+            clamp_value = torch.finfo(hidden_states.dtype).max - 1000
+            hidden_states = torch.clamp(hidden_states, min=-clamp_value, max=clamp_value)
+
+        outputs = (hidden_states,)
+
+        if use_cache:
+            outputs = outputs + (present_key_value_state,) + attention_outputs
+        else:
+            outputs = outputs + attention_outputs
+
+        return outputs  # hidden-states, present_key_value_states, (self-attention position bias), (self-attention weights), (cross-attention position bias), (cross-attention weights)
+
+
 class CustomT5Stack(T5Stack):
     def invert_attention_mask(self, encoder_attention_mask: Tensor) -> Tensor:
         if encoder_attention_mask.dim() == 3:
@@ -161,10 +268,11 @@ class PipelinedT5ForConditionalGeneration(
         self.encoder.__class__ = CustomT5Stack
         self.decoder.__class__ = CustomT5Stack
 
-        # Enable autocast for T5LayerNorm because computation cannot happen in fp16
-        for mod in self.modules():
-            if isinstance(mod, T5LayerNorm):
-                mod.forward = poptorch.autocast(enabled=True)(mod.forward)
+        # Use a custom T5Block implementation that removes a dynamic if blocks that can't be statically traced
+        for block in self.encoder.block:
+            block.__class__ = CustomT5Block
+        for block in self.decoder.block:
+            block.__class__ = CustomT5Block
 
         for index, layer in enumerate(self.encoder.block):
             ipu = layer_ipu[index]
@@ -209,9 +317,10 @@ class PipelinedT5ForConditionalGeneration(
         self.encoder.__class__ = T5Stack
         self.decoder.__class__ = T5Stack
 
-        for mod in self.modules():
-            if isinstance(mod, T5LayerNorm):
-                mod.forward = T5LayerNorm.forward.__get__(mod, T5LayerNorm)
+        for block in self.encoder.block:
+            block.__class__ = T5Block
+        for block in self.decoder.block:
+            block.__class__ = T5Block
 
         if self.ipu_config.embedding_serialization_factor > 1:
             old_lm_head = nn.Linear(
@@ -227,25 +336,25 @@ class PipelinedT5ForConditionalGeneration(
 
         return self
 
-    def _forward(
+    def forward(
         self,
-        input_ids=None,
-        attention_mask=None,
-        decoder_input_ids=None,
-        decoder_attention_mask=None,
-        head_mask=None,
-        decoder_head_mask=None,
-        cross_attn_head_mask=None,
-        encoder_outputs=None,
-        past_key_values=None,
-        inputs_embeds=None,
-        decoder_inputs_embeds=None,
-        labels=None,
-        use_cache=None,
-        output_attentions=None,
-        output_hidden_states=None,
-        return_dict=False,
-    ):
+        input_ids: Optional[torch.LongTensor] = None,
+        attention_mask: Optional[torch.FloatTensor] = None,
+        decoder_input_ids: Optional[torch.LongTensor] = None,
+        decoder_attention_mask: Optional[torch.BoolTensor] = None,
+        head_mask: Optional[torch.FloatTensor] = None,
+        decoder_head_mask: Optional[torch.FloatTensor] = None,
+        cross_attn_head_mask: Optional[torch.Tensor] = None,
+        encoder_outputs: Optional[Tuple[Tuple[torch.Tensor]]] = None,
+        past_key_values: Optional[Tuple[Tuple[torch.Tensor]]] = None,
+        inputs_embeds: Optional[torch.FloatTensor] = None,
+        decoder_inputs_embeds: Optional[torch.FloatTensor] = None,
+        labels: Optional[torch.LongTensor] = None,
+        use_cache: Optional[bool] = None,
+        output_attentions: Optional[bool] = None,
+        output_hidden_states: Optional[bool] = None,
+        return_dict: Optional[bool] = None,
+    ) -> Union[Tuple[torch.FloatTensor], Seq2SeqLMOutput]:
         use_cache = use_cache if use_cache is not None else self.config.use_cache
         return_dict = return_dict if return_dict is not None else self.config.use_return_dict
 
@@ -286,24 +395,10 @@ class PipelinedT5ForConditionalGeneration(
 
         hidden_states = encoder_outputs[0]
 
-        if self.model_parallel:
-            torch.cuda.set_device(self.decoder.first_device)
-
         if labels is not None and decoder_input_ids is None and decoder_inputs_embeds is None:
             # get decoder inputs from shifting lm labels to the right
             decoder_input_ids = self._shift_right(labels)
 
-        # Set device for model parallelism
-        if self.model_parallel:
-            torch.cuda.set_device(self.decoder.first_device)
-            hidden_states = hidden_states.to(self.decoder.first_device)
-            if decoder_input_ids is not None:
-                decoder_input_ids = decoder_input_ids.to(self.decoder.first_device)
-            if attention_mask is not None:
-                attention_mask = attention_mask.to(self.decoder.first_device)
-            if decoder_attention_mask is not None:
-                decoder_attention_mask = decoder_attention_mask.to(self.decoder.first_device)
-
         # Decode
         decoder_outputs = self.decoder(
             input_ids=decoder_input_ids,
@@ -324,7 +419,6 @@ class PipelinedT5ForConditionalGeneration(
 
         # Set device for model parallelism
         if self.model_parallel:
-            torch.cuda.set_device(self.encoder.first_device)
             self.lm_head = self.lm_head.to(self.encoder.first_device)
             sequence_output = sequence_output.to(self.lm_head.weight.device)
 
@@ -345,10 +439,15 @@ class PipelinedT5ForConditionalGeneration(
             loss = loss_fct(lm_logits.view(-1, lm_logits.size(-1)), labels.view(-1))
         # TODO(thom): Add z_loss https://github.com/tensorflow/mesh/blob/fa19d69eafc9a482aff0b59ddd96b025c0cb207d/mesh_tensorflow/layers.py#L666
 
+        # Only returning the loss to make the communication between the host and the device faster.
         if not return_dict:
             output = (lm_logits,) + decoder_outputs[1:] + encoder_outputs
-            return ((loss,) + output) if loss is not None else output
+            return (loss,) if labels is not None else output
 
+        if loss is not None:
+            return Seq2SeqLMOutput(
+                loss=loss,
+            )
         return Seq2SeqLMOutput(
             loss=loss,
             logits=lm_logits,
@@ -360,38 +459,3 @@ class PipelinedT5ForConditionalGeneration(
             encoder_hidden_states=encoder_outputs.hidden_states,
             encoder_attentions=encoder_outputs.attentions,
         )
-
-    def train(self, mode: bool = True) -> "PipelinedT5ForConditionalGeneration":
-        mod = super(T5ForConditionalGeneration, self).train(mode=mode)
-        # TODO: enable that once generation is supported.
-        # mod.forward = mod._forward_for_train if mode else mod._forward_for_generate
-        mod.forward = mod._forward_for_train
-        return mod
-
-    def _forward_for_train(self, input_ids, attention_mask, decoder_input_ids, labels=None):
-        outputs = self._forward(
-            input_ids=input_ids,
-            attention_mask=attention_mask,
-            decoder_input_ids=decoder_input_ids,
-            labels=labels,
-            use_cache=False,
-            return_dict=False,
-        )
-        # Only returning the loss to make the communication between the host and the device faster.
-        return outputs[0:1]
-
-    def _forward_for_generate(self, encoder_outputs, decoder_input_ids, attention_mask, labels=None):
-        outputs = super().forward(
-            encoder_outputs=encoder_outputs,
-            attention_mask=attention_mask,
-            decoder_input_ids=decoder_input_ids,
-            return_dict=False,
-            use_cache=False,
-            labels=labels,
-        )
-        # Only returning the loss (if labels is provided) and the logits.
-        if labels is None:
-            return outputs[:1]
-        return outputs[:2]
-
-    forward = _forward_for_train
